#include "kwlantraywin.h"
#include "wlanlib.h"

#include <kstdaction.h>
#include <kpopupmenu.h>
#include <kaction.h>
#include <qtooltip.h>
#include <kiconloader.h>
#include <qpixmap.h>
#include <klocale.h>






KWlanTrayWin::KWlanTrayWin(QWidget *parent, const char *name)
    : KSystemTray(parent, name)
{
    m_connected = false;
    // actionCollection()->remove( actionCollection()->action( "quit" ) );
    m_quit = KStdAction::quit( this, SIGNAL( quitSelected() ), actionCollection() );
    m_about = KStdAction::aboutApp( this, SIGNAL( aboutSelected() ), actionCollection() );
    setPixmap(loadIcon("kwlan/disconnected"));
    m_tooltip = new KWlanToolTip(this);
    m_profiles.setAutoDelete(true);
    m_profileMenu = new KPopupMenu();
    connect (m_profileMenu, SIGNAL(activated(int )),SLOT(selectProfile(int )));
    m_timer = new QTimer(this);
    if (m_timer){
        connect (m_timer, SIGNAL(timeout()),SLOT(updateIcon()));
        m_timer->start(5000,FALSE);
    }

}

KWlanTrayWin::~KWlanTrayWin() {
    delete m_tooltip;
    m_tooltip = 0;
}
 
void KWlanTrayWin::contextMenuAboutToShow( KPopupMenu *r ) {
    QString *prof;
    int id;
    r->clear();
    r->insertTitle(SmallIcon ("kwlan"), "KWlan - Wireless Lan Manager");
    id = 1;
    m_profileMenu->clear();
    if (!m_profiles.isEmpty()){
        prof = m_profiles.first();
        while (prof){
            m_profileMenu->insertItem(*prof,id);
            m_profileMenu->setItemParameter(id,id);
            prof=m_profiles.next();
            id++;
        }
        r->insertItem("Select Profile",m_profileMenu);
    }
    m_about->plug( r );
    m_quit->plug( r );
    
}

void KWlanTrayWin::clearProfiles()
{
    while (!m_profiles.isEmpty()) m_profiles.removeFirst();
}

void KWlanTrayWin::insertProfile(QString *profile)
{
    m_profiles.append(profile);
}


bool KWlanTrayWin::isConnected() const {
    return m_connected;
}
            
const QString & KWlanTrayWin::getInterface() const {
    return m_ifname;
}
        
const QString & KWlanTrayWin::getIpAddress() const {
    return m_ipaddress;
}


void KWlanTrayWin::updateIcon()
{
    //QPixmap trayIcon;
    if (!m_connected)
        setPixmap(loadIcon("kwlan/disconnected"));
    else if (!m_ifname.isEmpty()){
        IwLib iw(m_ifname);
        if (iw.getQuality() >35) setPixmap(loadIcon("kwlan/connected"));
        else setPixmap(loadIcon("kwlan/weakconnection"));
    }
}


void KWlanTrayWin::setInterface(const QString &ifname)
{
    IwLib iw(ifname);
    m_ifname = ifname;
    if (m_connected){
        if (iw.getQuality() >35) setPixmap(loadIcon("kwlan/connected"));
        else setPixmap(loadIcon("kwlan/weakconnection"));
    }
    else setPixmap(loadIcon("kwlan/disconnected"));

}

void KWlanTrayWin::setIpAddress(const QString &ipaddress){
    m_ipaddress = ipaddress;
}

void KWlanTrayWin::setConnectionStatus(bool status){
    m_connected=status;
    updateIcon();
}

void KWlanTrayWin::selectProfile(int id)
{
    emit profileChanged(m_profileMenu->text(id));
}

KWlanToolTip::KWlanToolTip(QWidget *parent): QToolTip(parent) {
}
    
KWlanToolTip::~KWlanToolTip() {
    
}
        
void KWlanToolTip::maybeTip(const QPoint &p) {
    QString tooltip;
    KWlanTrayWin *parent = (KWlanTrayWin *)parentWidget();
    const QString &ifname = parent->getInterface();
        
    // We can't show tooltip if no interface was specified
    if (!parent->rect().contains(p) || ifname.isEmpty())
        return;
         	
    if (parent->isConnected()) {
        IwLib iw(ifname);
                
        tooltip  = QString(i18n("Interface : <b>%1</b><br />")).arg(ifname);
        tooltip += QString(i18n("Connected to: <b>%1</b><br />")).arg(iw.getESSID());
        tooltip += QString(i18n("Bitrate: <b>%1</b><br />")).arg(iw.getBitrate());    
        tooltip += QString(i18n("Link quality: <b>%1%</b><br />")).arg(iw.getQuality());
        tooltip += QString(i18n("IP address: <b>%1</b>")).arg(parent->getIpAddress());
        	
    }
    else {
        tooltip  = QString(i18n("Interface : <b>%1</b><br />")).arg(ifname);
        tooltip += i18n("Disconnected");
    }
                 
    tip(parent->rect(), tooltip);
}
